<?php

namespace GirosGiroCheckout6\Helper;

use GirosGiroCheckout6\GirosGiroCheckout6;
use girosolution\GiroCheckout_SDK\GiroCheckout_SDK_Request;
use girosolution\GiroCheckout_SDK\helper\GiroCheckout_SDK_ResponseCode_helper;

/**
 * Class to manage a giropay payment method.
 *
 * Extends the AbstractPaymentProxy
 *
 * @author GiroSolution AG
 * @package GiroCheckout
 * @copyright Copyright (c) 2016, GiroSolution AG
 */
class GiropayPaymentProxy extends AbstractPaymentProxy
{
  private static $refINSTANCE = NULL;
  protected $container;
  protected $config;

  /**
   * Constructor method.
   * Set the payment type and payment method credentials.
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2016, GiroSolution AG
   * @return void
   */
  public function __construct($container, $systemConfigService)
  {
    $this->container = $container;
    $this->config = $systemConfigService;
    parent::__construct($container);
    self::$refINSTANCE = $this;
    $this->setPaymentMethod('gc_giropay');
    $this->pReadConfiguration();
  }

  /**
   * Process the payment transaction.
   * In this section a request method instance has to be passed and then is submits the request to the GiroCheckout API.
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2016, GiroSolution AG
   * @param int $amount The amount of order.
   * @param string $currency The shop currency.
   * @param string $transactionId The order number.
   * @param string $text The payment purpose.
   * @param string $bicGiropay The user bic.
   * @return array
   */
  public function startTransaction($amount = '', $currency = '', $transactionId = '', $text = '', $salesChannelContext = '')
  {
    $language = "de";
    $amount = isset($amount) ? (is_string($amount) ? trim($amount) : '') : '';
    $currency = isset($currency) ? (is_string($currency) ? trim($currency) : 'EUR') : 'EUR';
    $transactionId = isset($transactionId) ? (is_string($transactionId) ? trim($transactionId) : '') : '';
    $result = array();

    try {
      //Sends request to Girocheckout.
      $request = new GiroCheckout_SDK_Request('giropayTransaction');
      $request->setSecret($this->projectPassword);
      $request->addParam('merchantId', $this->merchantId)
          ->addParam('projectId', $this->projectId)
          ->addParam('merchantTxId', $transactionId)
          ->addParam('amount', $amount * 100)
          ->addParam('currency', $currency)
          ->addParam('purpose', $text)
          ->addParam('urlRedirect', $this->urlRedirect)
          ->addParam('urlNotify', $this->urlNotify)
          ->addParam('sourceId', $this->getGcSource())
          ->addParam('customerId', $salesChannelContext->getCustomer()->getId())
          ->submit();

      if ($request->requestHasSucceeded()) {
        $result["status"] = $request->getResponseParam('rc');
        $result["redirect"] = $request->getResponseParam('redirect');
      } else {
        $result["status"] = $request->getResponseParam('rc');
        $result["msg"] = GiroCheckout_SDK_ResponseCode_helper::getMessage($request->getResponseParam('rc'), $language);
      }
    } catch (\Exception $e) {
      $result["status"] = -1000;
      $result["msg"] = $e->getMessage();
    }
    return $result;
  }

  /**
   * Read the payment method credentials values from database.
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2016, GiroSolution AG
   * @return void
   */
  private function pReadConfiguration()
  {
    $config = $this->config->get('GirosGiroCheckout6.config');
    if (isset($config)) {
      if (isset($config['gcgiropayMerchantId'])) {
        $this->merchantId = trim((string)$config['gcgiropayMerchantId']);
      }

      if (isset($config['gcgiropayProjectId'])) {
        $this->projectId = trim((string)$config['gcgiropayProjectId']);
      }

      if (isset($config['gcgiropayProjectPassword'])) {
        $this->projectPassword = trim((string)$config['gcgiropayProjectPassword']);
      }

      // PaymentStateSuccess
      if (isset($config['gcgiropayPaymentStateSuccess'])) {
        $this->paymentStateSuccess = $this->getAsInt($config['gcgiropayPaymentStateSuccess']);
      }

      if (isset($config['gcgiropayPurpose'])) {
        $this->purpose = trim((string)$config['gcgiropayPurpose']);
      }

      // Email senden, wenn sich der Bezahlstatus aendert
      if (defined('GIROCONNECT_CONFIGURE_SEND_EMAIL') && GIROCONNECT_CONFIGURE_SEND_EMAIL === TRUE) {
        if (isset($config['gcgiropayStatusMail'])) {
          $value = $this->getAsBoolean($config['gcgiropayStatusMail']);
          $this->sendEmail = $value;
        }
      }
    }
  }

}
