<?php declare(strict_types=1);
namespace GirosGiroCheckout6\Helper;

use GirosGiroCheckout6\Service\PaymentGcbluecode;
use GirosGiroCheckout6\Service\PaymentGccreditcard;
use GirosGiroCheckout6\Service\PaymentGcdirectdebit;
use GirosGiroCheckout6\Service\PaymentGceps;
use GirosGiroCheckout6\Service\PaymentGcgiropay;
use GirosGiroCheckout6\Service\PaymentGcideal;
use GirosGiroCheckout6\Service\PaymentGcmaestro;
use GirosGiroCheckout6\Service\PaymentGcpaydirekt;
use GirosGiroCheckout6\Service\PaymentGcsofortuw;
use Shopware\Core\Content\Media\File\FileSaver;
use Shopware\Core\Content\Media\File\MediaFile;
use Shopware\Core\Content\Media\MediaService;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;
use Shopware\Core\Framework\Plugin\Context\ActivateContext;
use Shopware\Core\Framework\Plugin\Context\DeactivateContext;
use Shopware\Core\Framework\Plugin\Context\InstallContext;
use Shopware\Core\Framework\Plugin\Context\UninstallContext;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Shopware\Core\Framework\Uuid\Uuid;

class MediaInstaller
{
  private $mediaRepository;
  private $paymentMethodRepository;
  private $fileSaver;
  private $container;
  private $mediaService;

  public function __construct(
    ContainerInterface $container,
    MediaService $mediaService
  ) {
    $this->mediaRepository = $container->get('media.repository');
    $this->paymentMethodRepository = $container->get('payment_method.repository');
    $this->fileSaver = $container->get(FileSaver::class);
    $this->container = $container;
    $this->mediaService = $mediaService;
  }

  public function install(InstallContext $context): void
  {
    return;
  }

  public function uninstall(UninstallContext $context): void
  {
    return;
  }

  public function activate(ActivateContext $context): void
  {
    $payments = array(
      __DIR__  . "/../img/Logo_bluecode_40_px.png" => PaymentGcbluecode::class,
      __DIR__  . "/../img/visa_msc_amex_jcb_40px.png" => PaymentGccreditcard::class,
      __DIR__  . "/../img/Logo_EC_40_px.png" => PaymentGcdirectdebit::class,
      __DIR__  . "/../img/Logo_eps_40_px.jpg" => PaymentGceps::class,
      __DIR__  . "/../img/Logo_giropay_40_px.jpg" => PaymentGcgiropay::class,
      __DIR__  . "/../img/Logo_iDeal_40_px.jpg" => PaymentGcideal::class,
      __DIR__  . "/../img/Logo_maestro_40_px.png" => PaymentGcmaestro::class,
      __DIR__  . "/../img/Logo_giropay_paydirekt.png" => PaymentGcpaydirekt::class,
      __DIR__  . "/../img/Logo_sofort_40_px.png" => PaymentGcsofortuw::class,
    );

    foreach ($payments as $mediaPath => $class) {
      $this->addMedia($mediaPath, $class, $context->getContext());
    }
  }

  public function deactivate(DeactivateContext $context): void
  {
    $payments = array(
      __DIR__  . "/../img/Logo_bluecode_40_px.png" => PaymentGcbluecode::class,
      __DIR__  . "/../img/visa_msc_amex_jcb_40px.png" => PaymentGccreditcard::class,
      __DIR__  . "/../img/Logo_EC_40_px.png" => PaymentGcdirectdebit::class,
      __DIR__  . "/../img/Logo_eps_40_px.jpg" => PaymentGceps::class,
      __DIR__  . "/../img/Logo_giropay_40_px.jpg" => PaymentGcgiropay::class,
      __DIR__  . "/../img/Logo_iDeal_40_px.jpg" => PaymentGcideal::class,
      __DIR__  . "/../img/Logo_maestro_40_px.png" => PaymentGcmaestro::class,
      __DIR__  . "/../img/Logo_giropay_paydirekt.png" => PaymentGcpaydirekt::class,
      __DIR__  . "/../img/Logo_sofort_40_px.png" => PaymentGcsofortuw::class,
    );

    foreach ($payments as $mediaPath => $class) {
      $this->removeMedia($mediaPath, $context->getContext());
    }
  }

  private function addMedia($mediaPath, $paymentClass, $context): void
  {
    $filename = $this->getMediaName($mediaPath);
    if ($this->hasMediaAlreadyInstalled($mediaPath, $context)) {
      return;
    }

    //$mediaService = new MediaService;
    //$mediaId = $mediaService->createMediaInFolder('gsgirocheckout', $context, false);
    $mediaFile = $this->createMediaFile($mediaPath);
    $mediaId = $this->mediaService->createMediaInFolder('gsgirocheckout', $context, false);

    $this->mediaService->saveMediaFile(
      $mediaFile,
      $filename,
      $context,
      'gsgirocheckout',
      $mediaId
    );

    //$this->mediaRepository->create(
    //  [
    //    [
    //      'id' => $mediaId
    //    ]
    //  ],
    //  $context
    //);


    //$this->fileSaver->persistFileToMedia(
    //  $mediaFile,
    //  $this->getMediaName($mediaPath),
    //  $mediaId,
    //  $context
    //);

    //Attach logo to payment
    $result = $this->paymentMethodRepository->search(
      (new Criteria())->addFilter(new EqualsFilter(
        'handlerIdentifier',
        $paymentClass
      )),
      $context
    );
    $paymentMethodEntity = $result->getEntities()->first();
    $this->paymentMethodRepository->update([
      [
        'id' => $paymentMethodEntity->getId(),
        'mediaId' => $mediaId
      ]
    ], $context);
  }

  private function removeMedia($mediaPath, $context): void
  {
    $criteria = (new Criteria())->addFilter(
      new EqualsFilter(
        'fileName',
        $this->getMediaName($mediaPath)
      )
    );
    $media = $this->mediaRepository->search($criteria, $context)->first();
    $this->mediaRepository->delete(
      [
        [
          'id' => $media->getId()
        ]
      ],
      $context
    );

  }

  private function createMediaFile(string $filePath): MediaFile
  {
    return new MediaFile(
      $filePath,
      mime_content_type($filePath),
      pathinfo($filePath, PATHINFO_EXTENSION),
      filesize($filePath)
    );
  }


  private function hasMediaAlreadyInstalled($mediaPath, $context): bool
  {
    $criteria = (new Criteria())->addFilter(
      new EqualsFilter(
        'fileName',
        $this->getMediaName($mediaPath)
      )
    );

    $media = $this->mediaRepository->search($criteria, $context)->first();

    return $media ? true : false;
  }

  private function getMediaName($mediaPath): string
  {
    $str = trim(substr($mediaPath, strrpos($mediaPath, '/') + 1));
    return 'gcp_' . substr($str, 0, -4);
  }
}
