<?php declare(strict_types=1);

namespace GirosGiroCheckout6\Migration;

use Doctrine\DBAL\Connection;
use Shopware\Core\Defaults;
use Shopware\Core\Framework\Migration\MigrationStep;
use Shopware\Core\Framework\Uuid\Uuid;

class Migration1621955840 extends MigrationStep
{
    public function getCreationTimestamp(): int
    {
        return 1621955840;
    }

    public function update(Connection $connection): void
    {
      $sql = <<<SQL
CREATE TABLE IF NOT EXISTS `gc_user_pay_giroconnect_trx` (
                      `id` BINARY(16) NOT NULL,
                      `trxid` bigint unsigned NOT NULL auto_increment,
                      `gcreference` varchar(50) NULL,
                      `orderid` varchar(30) NULL,
                      `girocheckout_directdebit_mandate_reference` varchar(200) NULL,
                      `girocheckout_capture_reference` varchar(50),
                      `girocheckout_refund_reference` varchar(50),
                      `girocheckout_payment_name` varchar(50),
                      `status` integer NOT NULL default 5,
                      `userid` varchar(50) NULL,
                      `comment` mediumtext NULL,
                      `purpose` varchar(250) NULL,
                      `dispatchid` integer NULL,
                      `chargevat` integer NULL,
                      `created_at` DATETIME(3) NOT NULL,
                      `updated_at` DATETIME(3),
                      PRIMARY KEY (`trxid`)
                ) ENGINE=InnoDB  DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;
SQL;
      $connection->executeQuery($sql);
      $connection->transactional(function (Connection $connection) {
        $defaultFolderId = Uuid::randomBytes();
        $configurationId = Uuid::randomBytes();
        $result = $connection->executeQuery("SELECT * FROM media_default_folder WHERE entity = 'gsgirocheckout';");

        if ($result->rowCount()) {
          $connection->executeUpdate("DELETE FROM media_default_folder WHERE entity = 'gsgirocheckout';");
        }
        $connection->executeUpdate('
                INSERT INTO `media_default_folder` (`id`, `association_fields`, `entity`, `created_at`)
                VALUES (:id, :associationFields, :entity, :createdAt)
            ', [
          'id' => $defaultFolderId,
          'associationFields' => '[]',
          'entity' => 'gsgirocheckout',
          'createdAt' => (new \DateTime())->format(Defaults::STORAGE_DATE_TIME_FORMAT)
        ]);

        $connection->executeUpdate("
                INSERT INTO `media_folder_configuration`
                    (`id`, `thumbnail_quality`, `create_thumbnails`, `private`, created_at)
                VALUES
                    (:id, 80, 1, 0, :createdAt)
            ", [
          'id' => $configurationId,
          'createdAt' => (new \DateTime())->format(Defaults::STORAGE_DATE_TIME_FORMAT)
        ]);
        $result = $connection->executeQuery("SELECT * FROM media_folder WHERE name = 'GirosGiroCheckout6 Logos';");

        if ($result->rowCount()) {
          $connection->executeUpdate("DELETE FROM media_folder WHERE name = 'GirosGiroCheckout6 Logos';");
        }

        $connection->executeUpdate('
                INSERT INTO `media_folder`
                    (
                        `id`,
                        `name`,
                        `default_folder_id`,
                        `media_folder_configuration_id`,
                        `use_parent_configuration`,
                        `child_count`,
                        `created_at`
                    )
                VALUES
                    (:id, :folderName, :defaultFolderId, :configurationId, 0, 0, :createdAt)
            ', [
          'id' => Uuid::randomBytes(),
          'folderName' => 'GirosGiroCheckout6 Logos',
          'defaultFolderId' => $defaultFolderId,
          'configurationId' => $configurationId,
          'createdAt' => (new \DateTime())->format(Defaults::STORAGE_DATE_TIME_FORMAT),
        ]);
      });
    }

  public function updateDestructive(Connection $connection): void
    {
        // implement update destructive
    }
}
