const {Component} = Shopware;
import template from './sw-order.html.twig';
import deDE from '../../snippet/de_DE.json';
import enGB from '../../snippet/en_GB.json';

Shopware.Locale.extend('de-DE', deDE);
Shopware.Locale.extend('en-GB', enGB);
Component.override('sw-order-detail-base', {
    template,
    inject: ['GsgirocheckoutPaymentService'],
    data() {
        return {
            showCaptureModal: false,
            showRefundModal: false,
            showSuccessModal: false,
            showFailureModal: false,
            isCaptureSuccessful: false,
            isRefundSuccessful: false,
            allowCapture: false,
            allowRefund: false,
            isDirectdebittransaction: false,
        };
    },

    computed: {
        captureEnabled() {
            if (this.allowCapture) {
                return false;
            }
            return true;
        },

        refundEnabled() {
            if (this.allowRefund) {
                return false;
            }
            return true;
        },
    },

    methods: {

        openCaptureModal() {
            this.showCaptureModal = true;
            this.isSuccessful = false;
        },

        closeCaptureModal() {
            this.showCaptureModal = false;
        },

        openRefundModal() {
            this.showRefundModal = true;
            this.isSuccessful = false;
        },

        closeRefundModal() {
            this.showRefundModal = false;
        },

        openSuccessModal() {
            this.showFailureModal = false;
            this.showSuccessModal = true;
            this.isSuccessful = true;
        },
        closeSuccessModal() {
            this.showSuccessModal = false;
            this.isSuccessful = true;
        },
        openFailureModal() {
            this.showSuccessModal = false;
            this.showFailureModal = true;
            this.isSuccessful = true;
        },
        closeFailureModal() {
            this.showFailureModal = false;
            this.isSuccessful = false;
        },
        reloadPage() {
            window.location.reload();
        },

        getPluginName(transaction) {
            if (!transaction.customFields) {
                return false;
            }
            if (transaction.customFields['pluginName'] === 'gsgirocheckout') {
                return transaction.customFields['pluginName'];
            } else {
                return false;
            }
        },

        isCapturable(transaction) {
            if (!transaction.customFields) {
                return false;
            }
            return transaction.customFields['isCapturable'];

        },

        isRefundable(transaction) {
            if (!transaction.customFields) {
                return false;
            }
            return transaction.customFields['isRefundable'];

        },

        isDirectdebit(transaction) {
            if (!transaction.customFields) {
                return false;
            }
            return transaction.customFields['paymentName'];

        },

        isGiroCheckout(order) {
            let me = this;
            let bool = false;
            if (!order.transactions) {
                return false;
            }

            order.transactions.map(function (transaction) {
                if (!transaction.customFields) {
                    return false;
                }
                if (me.getPluginName(transaction)) {
                    if (me.isCapturable(transaction) === '1') {
                        me.allowCapture = true;;
                    }
                    if (me.isRefundable(transaction) === '1') {
                        me.allowRefund = true;
                    }
                    if (me.isDirectdebit(transaction) === 'gc_directdebit') {
                        me.isDirectdebittransaction = true;
                    }
                    if (me.allowCapture === true || me.allowRefund === true || me.isDirectdebittransaction === true) {
                        bool = true;
                    }
                }
            });
            return bool;
        },

        captureOrder(order) {
            let request;
            if (!order.transactions) {
                return false;
            }
            order.transactions.map(function (transaction) {
                request = {
                    paymentName: transaction.customFields['paymentName'],
                    amount: Math.round(transaction.amount.totalPrice * (10 ** order.currency.decimalPrecision), 0),
                    currency: order.currency.shortName,
                    transactionId: transaction.customFields['orderNumber'],
                    reference: transaction.customFields['gcReference']
                };
                if (transaction.customFields['paymentName'] === 'gc_directdebit') {;
                    request = {
                        paymentName: transaction.customFields['paymentName'],
                        amount: Math.round(transaction.amount.totalPrice * (10 ** order.currency.decimalPrecision), 0),
                        currency: order.currency.shortName,
                        transactionId: transaction.customFields['orderNumber'],
                        reference: transaction.customFields['gcReference'],
                        mandateReference: transaction.customFields['mandateReference']
                    };
                }
            });

            this.GsgirocheckoutPaymentService.capturePayment(request).then(() => {
                this.isCaptureSuccessful = true;
                this.allowCapture = false;
                this.allowRefund = true;
            }).catch((error) => {
                this.allowCapture = true;
                this.isCaptureSuccessful = false;
                console.log(error);
                this.openFailureModal();
            }).finally(() => {
                this.closeCaptureModal();
                this.openSuccessModal();
            });
        },

        refundOrder(order) {
            let request;
            if (!order.transactions) {
                return false;
            }
            order.transactions.map(function (transaction) {
                request = {
                    paymentName: transaction.customFields['paymentName'],
                    amount: Math.round(transaction.amount.totalPrice * (10 ** order.currency.decimalPrecision), 0),
                    currency: order.currency.shortName,
                    transactionId: transaction.customFields['orderNumber'],
                    reference: transaction.customFields['gcReference']
                };
                if (transaction.customFields['paymentName'] === 'gc_directdebit') {
                    request = {
                        paymentName: transaction.customFields['paymentName'],
                        amount: Math.round(transaction.amount.totalPrice * (10 ** order.currency.decimalPrecision), 0),
                        currency: order.currency.shortName,
                        transactionId: transaction.customFields['orderNumber'],
                        reference: transaction.customFields['gcReference'],
                        mandateReference: transaction.customFields['mandateReference']
                    };
                }
            });

            this.GsgirocheckoutPaymentService.refundPayment(request).then(() => {
                this.isRefundSuccessful = true;
                this.allowCapture = false;
                this.allowRefund = false;
            }).catch((error) => {
                this.allowRefund = true;
                this.isRefundSuccessful = false;
                console.log(error);
                this.openFailureModal();
            }).finally(() => {
                this.closeRefundModal();
                this.openSuccessModal();
            });
        },
    }
});
