<?php declare(strict_types=1);

namespace GirosGiroCheckout6\Storefront\Controller;


use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;
use Symfony\Component\Routing\RouterInterface;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionStateHandler;
use Shopware\Core\Framework\Routing\Annotation\RouteScope;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;
use Symfony\Component\Routing\Generator\UrlGeneratorInterface;
use Shopware\Core\System\SystemConfig\SystemConfigService;
use Symfony\Component\DependencyInjection\ContainerInterface;
use GirosGiroCheckout6\Helper\CreditcardPaymentProxy;
use GirosGiroCheckout6\Helper\DirectdebitPaymentProxy;
use GirosGiroCheckout6\Helper\PaydirektPaymentProxy;
use GirosGiroCheckout6\Helper\BluecodePaymentProxy;
use GirosGiroCheckout6\Helper\MaestroPaymentProxy;


class CaptureController extends AbstractController
{
  private $transactionStateHandler;
  public $router;
  private $orderRepository;
  protected $config;
  protected $container;

  public function __construct(
    OrderTransactionStateHandler $transactionStateHandler,
    UrlGeneratorInterface $router,
    EntityRepositoryInterface $orderRepository,
    SystemConfigService $systemConfigService,
    ContainerInterface $container
  )
  {
    $this->transactionStateHandler = $transactionStateHandler;
    $this->router = $router;
    $this->orderRepository = $orderRepository;
    $this->config = $systemConfigService;
    $this->container = $container;
  }

  /**
   * @RouteScope(scopes={"api"})
   * @Route("/api/v{version}/_action/gsgirocheckout/capture-payment", name="api.action.gsgirocheckout.capture_payment", methods={"POST"})
   */

  public function captureAction(Request $request): JsonResponse
  {
    $data = $request->request;
    $orderRepository = $this->container->get('order.repository');
    $transactionRepository = $this->container->get('order_transaction.repository');
    $paymentName = $data->get('paymentName');
    $amount = $data->get('amount');
    $currency = $data->get('currency');
    $transactionId = $data->get('transactionId');
    $reference = $data->get('reference');
    $userId = $data->get('userId');
    $criteria = new Criteria();
    $criteria->addFilter(new EqualsFilter('orderNumber', $transactionId))->addAssociation('transactions');
    $order = $orderRepository->search($criteria, Context::createDefaultContext())->last();
    $iOrderId = $order->getId();
    $criteria = new Criteria();
    $criteria->addFilter(new EqualsFilter('orderId', $iOrderId));
    $transaction = $transactionRepository->search($criteria, Context::createDefaultContext())->last();
    $iTrxId = $transaction->getId();

    switch ($paymentName) {
      case "gc_creditcard":
        $paymentProxy = new CreditcardPaymentProxy($this->container, $this->config);
        $response = $paymentProxy->capturePayment($amount, $currency, $transactionId, $reference, $userId);

        if ($response['status'] == "OK") {
          if (method_exists($this->transactionStateHandler, 'paid')) {
            $this->transactionStateHandler->paid($iTrxId, Context::createDefaultContext());
          } else {
            $this->transactionStateHandler->pay($iTrxId, Context::createDefaultContext());
          }
          return new JsonResponse(['status' => $response['status'], 'message' => $response['message'], 'reference' => $response['reference']], Response::HTTP_ACCEPTED);
        } else if ($response['status'] == "ERROR") {
          return new JsonResponse(['status' => $response['status'], 'message' => $response['message'], 'reference' => $response['reference']], Response::HTTP_BAD_REQUEST);
        }

        return new JsonResponse(['status' => $response['status'], 'message' => $response['message'], 'reference' => $response['reference']], Response::HTTP_GATEWAY_TIMEOUT);
        break;

      case "gc_directdebit":
        $paymentProxy = new DirectdebitPaymentProxy($this->container, $this->config);
        $response = $paymentProxy->capturePayment($amount, $currency, $transactionId, $reference, $userId);

        if ($response['status'] == "OK") {
          if (method_exists($this->transactionStateHandler, 'paid')) {
            $this->transactionStateHandler->paid($iTrxId, Context::createDefaultContext());
          } else {
            $this->transactionStateHandler->pay($iTrxId, Context::createDefaultContext());
          }
          return new JsonResponse(['status' => $response['status'], 'message' => $response['message'], 'reference' => $response['reference']], Response::HTTP_ACCEPTED);
        } else if ($response['status'] == "ERROR") {
          return new JsonResponse(['status' => $response['status'], 'message' => $response['message'], 'reference' => $response['reference']], Response::HTTP_BAD_REQUEST);
        }

        return new JsonResponse(['status' => $response['status'], 'message' => $response['message'], 'reference' => $response['reference']], Response::HTTP_GATEWAY_TIMEOUT);
        break;
      case "gc_bluecode":
        $paymentProxy = new BluecodePaymentProxy($this->container, $this->config);
        $response = $paymentProxy->capturePayment($amount, $currency, $transactionId, $reference, $userId);

        if ($response['status'] == "OK") {
          if (method_exists($this->transactionStateHandler, 'paid')) {
            $this->transactionStateHandler->paid($iTrxId, Context::createDefaultContext());
          } else {
            $this->transactionStateHandler->pay($iTrxId, Context::createDefaultContext());
          }
          return new JsonResponse(['status' => $response['status'], 'message' => $response['message'], 'reference' => $response['reference']], Response::HTTP_ACCEPTED);
        } else if ($response['status'] == "ERROR") {
          return new JsonResponse(['status' => $response['status'], 'message' => $response['message'], 'reference' => $response['reference']], Response::HTTP_BAD_REQUEST);
        }

        return new JsonResponse(['status' => $response['status'], 'message' => $response['message'], 'reference' => $response['reference']], Response::HTTP_GATEWAY_TIMEOUT);
        break;

      case "gc_paydirekt":
        $paymentProxy = new PaydirektPaymentProxy($this->container, $this->config);
        $response = $paymentProxy->capturePayment($amount, $currency, $transactionId, $reference, $userId);

        if ($response['status'] == "OK") {
          if (method_exists($this->transactionStateHandler, 'paid')) {
            $this->transactionStateHandler->paid($iTrxId, Context::createDefaultContext());
          } else {
            $this->transactionStateHandler->pay($iTrxId, Context::createDefaultContext());
          }
          return new JsonResponse(['status' => $response['status'], 'message' => $response['message'], 'reference' => $response['reference']], Response::HTTP_ACCEPTED);
        } else if ($response['status'] == "ERROR") {
          return new JsonResponse(['status' => $response['status'], 'message' => $response['message'], 'reference' => $response['reference']], Response::HTTP_BAD_REQUEST);
        }

        return new JsonResponse(['status' => $response['status'], 'message' => $response['message'], 'reference' => $response['reference']], Response::HTTP_GATEWAY_TIMEOUT);
        break;

      case "gc_maestro":
        $paymentProxy = new MaestroPaymentProxy($this->container, $this->config);
        $response = $paymentProxy->capturePayment($amount, $currency, $transactionId, $reference, $userId);

        if ($response['status'] == "OK") {
          if (method_exists($this->transactionStateHandler, 'paid')) {
            $this->transactionStateHandler->paid($iTrxId, Context::createDefaultContext());
          } else {
            $this->transactionStateHandler->pay($iTrxId, Context::createDefaultContext());
          }
          return new JsonResponse(['status' => $response['status'], 'message' => $response['message'], 'reference' => $response['reference']], Response::HTTP_ACCEPTED);
        } else if ($response['status'] == "ERROR") {
          return new JsonResponse(['status' => $response['status'], 'message' => $response['message'], 'reference' => $response['reference']], Response::HTTP_BAD_REQUEST);
        }

        return new JsonResponse(['status' => $response['status'], 'message' => $response['message'], 'reference' => $response['reference']], Response::HTTP_GATEWAY_TIMEOUT);
        break;

    }
    return new JsonResponse(['status' => 'ERROR', 'message' => 'error in captureAction'], Response::HTTP_BAD_REQUEST);
  }
}