<?php declare(strict_types=1);

namespace GirosGiroCheckout6\Storefront\Controller;

use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Generator\UrlGeneratorInterface;
use Shopware\Core\System\SystemConfig\SystemConfigService;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Routing\Annotation\Route;
use Shopware\Core\Framework\Routing\Annotation\RouteScope;
use girosolution\GiroCheckout_SDK\GiroCheckout_SDK_Tools;
use girosolution\GiroCheckout_SDK\GiroCheckout_SDK_Request;
use girosolution\GiroCheckout_SDK\helper\GiroCheckout_SDK_TransactionType_helper;


class ConfigurationController extends AbstractController
{
  public $router;
  protected $config;
  protected $container;

  public function __construct(
      UrlGeneratorInterface $router,
      SystemConfigService $systemConfigService,
      ContainerInterface $container

  )
  {
    $this->router = $router;
    $this->config = $systemConfigService;
    $this->container = $container;
  }

  /**
   * @RouteScope(scopes={"api"})
   * @Route("/api/v{version}/_action/gsgirocheckout/validate-keys", name="api.action.gsgirocheckout.validate_keys", methods={"POST"})
   */

  public function validateKeys(Request $request): JsonResponse
  {
    $data = $request->request;
    $keys = $data->get('keys');
    $strErrInfo = '';

    $merchantIdAux = '';
    $projectIdAux = '';
    $projectPasswordAux = '';

    $resultArray = [];
    $paymentMethodToTest = '';


    foreach ($keys as $key => $value){
      if(strpos($key, 'MerchantId')){
        $merchantIdAux = $value;
      }else if(strpos($key, 'ProjectId')){
        $projectIdAux = $value;
      }else if(strpos($key, 'ProjectPassword')){
        $projectPasswordAux = $value;
      }

      switch (true){
         case stristr($key,'giropay'):
            $paymentMethodToTest = 'giropayTransaction';
            break;
         case stristr($key,'creditcard'):
            $paymentMethodToTest = 'creditCardTransaction';
            break;
         case stristr($key,'directdebit'):
            $paymentMethodToTest = 'directDebitTransaction';
            break;
         case stristr($key,'eps'):
            $paymentMethodToTest = 'epsTransaction';
            break;
         case stristr($key,'ideal'):
            $paymentMethodToTest = 'idealPayment';
            break;
         case stristr($key,'bluecode'):
            $paymentMethodToTest = 'blueCodeTransaction';
            break;
         case stristr($key,'maestro'):
           $paymentMethodToTest = 'maestroTransaction';
            break;
         case stristr($key,'paydirekt'):
           $paymentMethodToTest = 'paydirektTransaction';
            break;
         case stristr($key,'sofortuw'):
           $paymentMethodToTest = 'sofortuwTransaction';
            break;
      }

      if(!empty($merchantIdAux) && !empty($projectIdAux) && !empty($projectPasswordAux)){
        $bIsValid = GiroCheckout_SDK_Tools::testApiCredentials( $paymentMethodToTest, $merchantIdAux, $projectIdAux, $projectPasswordAux, $strErrInfo );
        if($bIsValid){
          $resultArray[$paymentMethodToTest] = 'OK';
          $merchantIdAux = "";
          $projectIdAux = "";
          $projectPasswordAux = "";
        }else{
          $resultArray[$paymentMethodToTest] = 'ERROR';
          $merchantIdAux = "";
          $projectIdAux = "";
          $projectPasswordAux = "";
        }
      }
    }
     return new JsonResponse([
            'results' => $resultArray
        ]);
  }
}