<?php declare(strict_types=1);

namespace GirosGiroCheckout6\Storefront\Controller;

use Exception;
use GirosGiroCheckout6\Service\LoggerService;
use Shopware\Core\Checkout\Order\OrderEntity;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionStateHandler;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;
use Shopware\Core\Framework\DataAbstractionLayer\Exception\InconsistentCriteriaIdsException;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;
use Shopware\Storefront\Controller\StorefrontController;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\Routing\RouterInterface;
use Symfony\Component\Routing\Annotation\Route;
use Shopware\Core\Framework\Routing\Annotation\RouteScope;
use Symfony\Component\Routing\Generator\UrlGeneratorInterface;
use Shopware\Core\System\SystemConfig\SystemConfigService;
use GirosGiroCheckout6\Helper\AbstractPaymentProxy;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Response;
use Shopware\Core\System\SalesChannel\SalesChannelContext;
use Shopware\Storefront\Page\GenericPageLoader;
use Shopware\Storefront\Page\Account\Order\AccountEditOrderPage;
use Shopware\Core\Framework\Context;
use girosolution\GiroCheckout_SDK\helper\GiroCheckout_SDK_ResponseCode_helper;


class ReturnController extends StorefrontController
{
  public $router;
  private $orderRepository;
  private $transactionStateHandler;
  private $context;
  protected $config;
  protected $container;
  private $genericLoader;
  private $logger;

  public function __construct(
    UrlGeneratorInterface $router,
    EntityRepositoryInterface $orderRepository,
    SystemConfigService $systemConfigService,
    ContainerInterface $container,
    OrderTransactionStateHandler $transactionStateHandler,
    GenericPageLoader $genericLoader,
    LoggerService $logger
  )

  {
    $this->router = $router;
    $this->orderRepository = $orderRepository;
    $this->config = $systemConfigService;
    $this->container = $container;
    $this->transactionStateHandler = $transactionStateHandler;
    $this->genericLoader   = $genericLoader;
    $this->logger = $logger;
  }

  /**
   * @RouteScope(scopes={"storefront"})
   * @Route("/gsgirocheckout/returnAction/{strPayment}",
   *      name="frontend.gsgirocheckout.returnAction",
   *      options={"seo"="false"},
   *      methods={"GET", "POST"}
   *     )
   */
  public function returnAction(string $strPayment, Request $request, SalesChannelContext $context): ?Response
  {
    try {

      $paymentProxy = new AbstractPaymentProxy($this->container);
      $config = $this->config->get('GirosGiroCheckout6.config');
      switch ($strPayment) {
        case "gc_creditcard":
          $return = $paymentProxy->notifyAction($request->query->all(), 'creditCardTransaction', $config['gccreditcardProjectPassword'],
            "gc_creditcard", $config['gccreditcardTransactionType'], $this->transactionStateHandler);
          break;
        case "gc_directdebit":
          $return = $paymentProxy->notifyAction($request->query->all(), 'directDebitTransaction', $config['gcdirectdebitProjectPassword'],
            "gc_directdebit", $config['gcdirectdebitTransactionType'], $this->transactionStateHandler);
          break;

        case "gc_bluecode":
          $return = $paymentProxy->notifyAction($request->query->all(), 'blueCodeTransaction', $config['gcbluecodeProjectPassword'],
            "gc_bluecode", $config['gcbluecodeTransactionType'], $this->transactionStateHandler);
          break;

        case "gc_ideal":
          $return = $paymentProxy->notifyAction($request->query->all(), 'idealPayment', $config['gcidealProjectPassword'],
            "gc_ideal", 'SALE', $this->transactionStateHandler);
          break;

        case "gc_eps":
          $return = $paymentProxy->notifyAction($request->query->all(), 'epsTransaction', $config['gcepsProjectPassword'],
            "gc_eps", 'SALE', $this->transactionStateHandler);
          break;

        case "gc_sofortuw":
          $return = $paymentProxy->notifyAction($request->query->all(), 'sofortuwTransaction', $config['gcsofortuwProjectPassword'],
            "gc_sofortuw", 'SALE', $this->transactionStateHandler);
          break;

        case "gc_paydirekt":
          $return = $paymentProxy->notifyAction($request->query->all(), 'paydirektTransaction', $config['gcpaydirektProjectPassword'],
            "gc_paydirekt", $config['gcpaydirektTransactionType'], $this->transactionStateHandler);
          break;

        case "gc_maestro":
          $return = $paymentProxy->notifyAction($request->query->all(), 'maestroTransaction', $config['gcmaestroProjectPassword'],
            "gc_maestro", $config['gcmaestroTransactionType'], $this->transactionStateHandler);
          break;

        case "gc_giropay":
          $return = $paymentProxy->notifyAction($request->query->all(), 'giropayTransaction', $config['gcgiropayProjectPassword'],
            "gc_giropay", 'SALE', $this->transactionStateHandler);
          break;

      }
      if($request->query->get('gcResultPayment') == 4000){
        $order = $this->getOrderFromNumber($request->query->get('gcMerchantTxId'));
        return $this->redirectToRoute('frontend.checkout.finish.page', ['orderId' => $order->getId()]);
      }
    }
    catch (Exception $e) {
      $this->logger->addEntry(
        sprintf('Error in ReturnController. '),
        $context->getContext(),
        $e,
        [
          'Error Message' => $e->getMessage(),
        ]
      );
    }

    $order = $this->getOrderFromNumber($request->query->get('gcMerchantTxId'));
    if (version_compare($this->container->getParameter('kernel.shopware_version'), '6.3.3.0') >= 0) {
      return $this->redirectToRoute('frontend.account.edit-order.page', [
        'orderId' => $order->getId(),
        'paymentFailed' => true,
        'error-code' => 'CHECKOUT__CUSTOMER_CANCELED_EXTERNAL_PAYMENT',
      ]);
    }
    else {
      $page = $this->genericLoader->load($request, $context);
      $errorMsg = GiroCheckout_SDK_ResponseCode_helper::getMessage($request->query->get('gcResultPayment'), 'DE');
      return $this->renderStorefront('@Storefront/storefront/page/checkout/payment/canceled.html.twig', [
        'page' => $page,
        'error' => $errorMsg,
        'returnUrl' => $this->router->generate('frontend.account.edit-order.page',['orderId' => $order->getId()] )
      ]);
    }

  }
  /**
   * @RouteScope(scopes={"storefront"})
   * @Route("/gsgirocheckout/errorAction/{errorMsg}",
   *      name="frontend.gsgirocheckout.errorAction",
   *      options={"seo"="false"},
   *      methods={"GET", "POST"}
   *     )
   */
  public function errorAction(string $errorMsg, Request $request, SalesChannelContext $context): ?Response
  {
    $page = $this->genericLoader->load($request, $context);
    return $this->renderStorefront('@Storefront/storefront/page/checkout/payment/fail.html.twig', [
      'errorMsg' => $errorMsg,
      'page' => $page,
    ]);
  }

  /**
   * @param string $orderNumber
   * @return OrderEntity
   * @throws InconsistentCriteriaIdsException
   */
  public function getOrderFromNumber(string $orderNumber): OrderEntity
  {
    $orderRepo = $this->orderRepository;
    $criteria = new Criteria();
    $criteria->addFilter(new EqualsFilter('orderNumber', $orderNumber))
      ->addAssociation('transactions');
    return $orderRepo->search($criteria, Context::createDefaultContext())->first();
  }

  /**
   * @return array
   */
  public function getReturnUrl($strPayment): string
  {
    return $this->router->generate('frontend.gsgirocheckout.returnAction',
      ['strPayment' => $strPayment],
      UrlGeneratorInterface::ABSOLUTE_URL
    );
  }


  public function getErrorUrl($errorMsg): string
  {
    return $this->router->generate('frontend.gsgirocheckout.errorAction',
      ['errorMsg' => $errorMsg],
      UrlGeneratorInterface::ABSOLUTE_URL
    );
  }
}
